from typing import Text

from pydantic.typing import Literal

TLP_ALLOWED_VALUES = ("RED", "AMBER", "GREEN", "WHITE")

TLP = Literal[TLP_ALLOWED_VALUES]


def valid_tlp(tlp: Text) -> bool:
    return tlp in TLP_ALLOWED_VALUES



"""
    Support for vocularies with support for
    - aliases
    - automatic generate regular expression from aliases
    - NLP stemming
"""


import configparser
import re
import sys
from logging import info
from typing import Any, Callable, Dict, List, Optional, Pattern, Union

import addict
import nltk  # type: ignore

import act.scio.aliasregex as aliasregex
from act.scio.alias import parse_aliases

DEFAULT_CONFIG = addict.Dict(
    {
        "alias": None,
        "default": None,
        "regexfromalias": False,
        "primary": False,
        "key_mod": "lower",
        "regexmanual": "",
        "object_type": None,
    }
)


def identity(x: Any) -> Any:
    """Identity function. Used as default to return same value as input"""
    return x


def from_config(config_filename: str) -> addict.Dict:
    """
    Load vocabularies from config and return Dict
    with vocabulary-name as key and Vocabulary as value
    """
    cparser = configparser.ConfigParser()
    cparser.read(str(config_filename))

    vocabularies = addict.Dict()

    for vocabulary_name in cparser.sections():
        section = cparser[vocabulary_name]

        # Config
        config = addict.Dict()
        config.alias = section.get("alias", DEFAULT_CONFIG.alias)
        config.default = section.get("default", DEFAULT_CONFIG.default)
        config.regexfromalias = section.getboolean("regexfromalias")
        config.primary = section.getboolean("primary")
        config.key_mod = section.get("key_mod", DEFAULT_CONFIG.key_mod)
        config.regexmanual = section.get("regexmanual", DEFAULT_CONFIG.regexmanual)
        config.object_type = section.get("object_type", DEFAULT_CONFIG.object_type)
        vocabularies[vocabulary_name] = Vocabulary(config)

    return vocabularies


class IllegalVocabularyKeyType(Exception):
    """Non existing Vocabulary Key Type"""

    pass


class Vocabulary:
    """
    Support for vocularies with support for
    - aliases
    - automatic generate regular expression from aliases
    - NLP stemming
    """

    def __init__(
        self, config: Union[addict.Dict, Dict, configparser.SectionProxy]
    ) -> None:
        """
        Args:
            config (addict.Dict):  addict.Dict with the following keys
                    * alias (str)            # Config filw with aliases
                    * primary (bool)         # Point aliases to primary name
                    * default (bool)         # Default value for if key does not exist
                    * regexfromalias (bool)  # Create regex from aliases?
                    * primary (bool)         # Use primary name as default return value
                    * key_mod (str)          # Default key_mod (default = "lower")
                    * regexmanual (str[])    # Extra regular expressions for regex search
                    * object_type (str)      # ACT object type applicable for this vocabulary
        """
        self.config = addict.Dict(DEFAULT_CONFIG)
        self.config.update(config)
        self.regex: List[Pattern] = []
        self.vocab: Dict[str, Dict[str, addict.Dict]] = addict.Dict(
            none=addict.Dict(),
            lower=addict.Dict(),
            stem=addict.Dict(),
            norm=addict.Dict(),
        )

        self.stemmer = nltk.stem.PorterStemmer().stem

        if self.config.alias:
            self.load_alias(self.config.alias)  # type: ignore

        if self.config.regexmanual:
            self.regex = [
                re.compile(entry.strip(), re.I)
                for entry in self.config.regexmanual.split("\n")
                if entry.strip()
            ]
        else:
            self.regex = []

        if self.config.regexfromalias:
            for aliasre in aliasregex.get_reg_ex_set(self.config.alias):
                try:
                    self.regex.append(re.compile(aliasre, re.IGNORECASE))
                except re.error:
                    sys.stderr.write(f"ERROR in regex {aliasre}\n")
                    raise

    def load_alias(self, filename: str) -> None:
        """
        Load aliases from file.

        Args:
            filename (str): Filename to read aliases from
        """
        with open(filename, "r") as f:

            for line in f.readlines():
                # Remove comments (starting with '#'), unless they are escaped
                line = re.sub(r"(?<!\\)#.*", "", line)

                if not line.strip():
                    continue  # Skip empty lines

                # Get key and aliases
                primary, aliases = parse_aliases(line)

                for name in [primary] + aliases:
                    # Stor a separate entry for each type of key_mod
                    # For each entry, also store the value itself and the
                    # primary name

                    self.vocab["none"][name] = addict.Dict(value=name, primary=primary)
                    self.vocab["lower"][name.lower()] = addict.Dict(
                        value=name, primary=primary
                    )
                    self.vocab["stem"][self.stemmer(name)] = addict.Dict(
                        value=name, primary=primary
                    )

                    self.vocab["norm"][aliasregex.normalize(name)] = addict.Dict(
                        value=name, primary=primary
                    )

    def __getitem__(self, key: str) -> Optional[str]:
        """
        Return entry from vocabulary.
        """

        return self.get(key)

    def regex_search(
        self,
        text,
        key_mod: str = "DEFAULT",
        normalize_result: Callable = identity,
        debug: bool = False,
    ) -> List[str]:
        """
        Find all matches in vocabulary from regex search

        Args:
            text (str):       Input text
            key_mod (text):   Lookup using modifie key
        """

        key_mod = self.get_key_mod(key_mod)

        result = []
        for regex in self.regex:
            for match in regex.findall(text):
                if debug:
                    logging.info("%s found by regex %s", match, regex)
                result.append(normalize_result(match))

        return result

    def get_key_mod(self, key_mod: Optional[str]) -> str:
        """Verify and get key_mod"""

        # Get key modifier from config
        if key_mod == "DEFAULT":
            key_mod = self.config.key_mod

        elif key_mod is None:  # Use "none" modifier (the value itself)
            key_mod = "none"

        if key_mod not in self.vocab:
            raise IllegalVocabularyKeyType("Illegal key: {}".format(key_mod))
        return key_mod

    def get(
        self,
        key: str,
        key_mod: str = "DEFAULT",
        primary: Optional[bool] = None,
        default: Optional[str] = None,
    ) -> Optional[str]:
        """
        Search for entry in vocabulary

        Args:
            key (str):           Key to search for
            key_mod (str):       Key modifier ("lower", "stem" or "none"). If not set,
                                 use default value form self.config.key_mod.
            primary (bool|None): If True, return primary name, if not return the
                                 value itself from the vocabulary
            default (str|None):  Default value if key is not found. Default == None
        """

        key_mod = self.get_key_mod(key_mod)

        # If primary is not set, get from config whether we should retrieve the primary name
        if primary is None:
            primary = self.config.primary

        # If default is None, get default value from config
        if default is None:
            default = self.config.default

        if key_mod == "stem":
            key = self.stemmer(key)
        elif key_mod == "lower":
            key = key.lower()
        elif key_mod == "norm":
            key = aliasregex.normalize(key)

        value = self.vocab[key_mod].get(key)

        if not value:
            return default

        if primary:  # Return primary value
            return value.primary  # type: ignore

        # Return value ifself
        return value.value  # type: ignore



"""plugin module to extract MITRE Att&ck IDs from documents"""

import re
from typing import List, Text

import addict

from act.scio.plugin import BasePlugin, Result


class Plugin(BasePlugin):  # pylint: disable=r0903
    """Scio2 plugin"""

    name = "mitre_attack"
    info = "References to MITRE Att&ck IDs"
    version = "0.1"
    dependencies: List[Text] = []

    tactic_re = re.compile(r"\bTA[0-9]{4}\b")
    technique_re = re.compile(r"\bT[0-9]{4}(?!\.[0-9]{3})\b")
    subtechnique_re = re.compile(r"\bT[0-9]{4}\.[0-9]{3}\b")
    software_re = re.compile(r"\bS[0-9]{4}\b")
    group_re = re.compile(r"\bG[0-9]{4}\b")

    async def analyze(self, nlpdata: addict.Dict) -> Result:

        res = addict.Dict()

        res.Groups = self.group_re.findall(nlpdata.content)
        res.Tactics = self.tactic_re.findall(nlpdata.content)
        res.Techniques = self.technique_re.findall(nlpdata.content)
        res.SubTechniques = self.subtechnique_re.findall(nlpdata.content)
        res.Software = self.software_re.findall(nlpdata.content)

        return Result(name=self.name, version=self.version, result=res)



"""Copyright 2020 mnemonic AS <opensource@mnemonic.no>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

---
Config utilities for scio
"""

import argparse
import logging
import os
from typing import Optional, Text

import caep
import elasticsearch
import greenstalk  # type: ignore

import act.scio.es


def parse_args(description: Text) -> argparse.ArgumentParser:
    """Parse default arguments"""
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument(
        "--beanstalk",
        help="Connect to beanstalk server. " + "If not specified, read from stdin",
        default="localhost",
    )
    parser.add_argument(
        "--beanstalk-port", type=int, default=11300, help="Default 11300"
    )
    parser.add_argument("--elasticsearch", help="Elasticsearch host to connect to.")
    parser.add_argument("--elasticsearch-user", help="Elasticsearch user")
    parser.add_argument("--elasticsearch-password", help="Elasticsearch password")
    parser.add_argument(
        "--elasticsearch-port", type=int, default=9200, help="Default 9200"
    )
    parser.add_argument(
        "--config-dir",
        default=caep.get_config_dir("scio"),
        help="Default config dir with configurations for scio and plugins",
    )
    parser.add_argument("--logfile")
    parser.add_argument("--loglevel", default="info")

    return parser


def beanstalk_client(
    args: argparse.Namespace, use: Optional[Text] = None, watch: Optional[Text] = None
) -> Optional[greenstalk.Client]:
    """Return beanstalk client if args.beanstalk, otherwise, return None"""
    client = None
    if args.beanstalk:
        logging.info("Connection to beanstalk")
        client = greenstalk.Client((args.beanstalk, args.beanstalk_port), encoding=None)
        if watch:
            client.watch(watch)
        if use:
            client.use(use)

    return client


def elasticsearch_client(
    args: argparse.Namespace,
) -> Optional[elasticsearch.client.Elasticsearch]:
    """Return elasticsearch client if args.elasticsearch, otherwise, return None"""
    if args.elasticsearch:
        logging.info("Connection to elasticsearch")
        return act.scio.es.es_client(
            host=args.elasticsearch,
            port=args.elasticsearch_port,
            username=args.elasticsearch_user,
            password=args.elasticsearch_password,
        )
    return None


def get_cache_dir(cache_id: str, create: bool = False) -> str:
    """
        Get cache dir.

        Honors $XDG_CACHE_HOME, but fallbacks to $HOME/.cache

    Args:
        cache_id [str]: directory under CACHE that will be used
        create [bool]: create directory if not exists

    Return path to cache_directory
    """

    home = os.environ["HOME"]

    cache_home = os.environ.get("XDG_CACHE_HOME", os.path.join(home, ".cache"))
    cache_dir = os.path.join(cache_home, cache_id)

    if create and not os.path.isdir(cache_dir):
        os.makedirs(cache_dir)

    return cache_dir



#!/usr/bin/env python3
""" Handle worker config"""


import argparse
import os
import sys
from typing import List, Text, Tuple, Union

import caep
from pkg_resources import (
    Requirement,
    resource_exists,
    resource_isdir,
    resource_listdir,
    resource_string,
)

CONFIG_ID = "scio"
CONFIG_NAME = "scio.ini"

PKG_NAME = "act.scio"


# there is no isfile in the pkg_resources.resource_* so we use the inverse of isdir
def resource_isfile(*x: Union[str, Requirement]) -> bool:
    return not resource_isdir(*x)  # type: ignore


def parseargs() -> argparse.Namespace:
    """Parse arguments"""

    parser = argparse.ArgumentParser(
        "ACT SCIO config",
        epilog="""
    show - Print default config
    user - Copy default config to {0}/{1}
    system - Copy default config to /etc/{1}
""".format(
            caep.get_config_dir(CONFIG_ID), CONFIG_NAME
        ),
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument("action", nargs=1, choices=["show", "user", "system"])

    return parser.parse_args()


def default_ini() -> List[Tuple[Text, Text]]:
    "Get content of default ini file"

    files: List[Tuple[Text, Text]] = []

    for resource_dir in ["etc", "etc/plugins", "vendor"]:
        if resource_exists(PKG_NAME, resource_dir) and resource_isdir(
            PKG_NAME, resource_dir
        ):
            for fname in [
                x
                for x in resource_listdir(PKG_NAME, resource_dir)
                if resource_isfile(PKG_NAME, x)
            ]:
                try:
                    files.append(
                        (
                            f"{resource_dir}/{fname}",
                            resource_string(
                                PKG_NAME, f"{resource_dir}/" + fname
                            ).decode("utf-8"),
                        )
                    )
                except UnicodeDecodeError as err:
                    sys.stderr.write(f"WARNING: Skipping file {fname} [{err}]\n")

    return files


def make_path(configdir: Text, path: Text) -> None:
    """Make sure the config directories exists before storing files."""
    plugin_path = os.path.join(configdir, path)
    try:
        os.mkdir(plugin_path)
        print(f"creating directory {plugin_path}")
    except FileExistsError:
        print(f"WARNING: config path allready exists {plugin_path}")
    except PermissionError:
        sys.stderr.write(f"ERROR: Permission denied when creating {plugin_path}\n")
        sys.exit(2)


def save_config(configdir: Text) -> None:
    """Save config to specified directory"""

    # Make sure the config directories exists before storing files.
    make_path(configdir, "etc")
    make_path(configdir, "etc/plugins")
    make_path(configdir, "vendor")

    for filename, content in default_ini():
        full_filename = os.path.join(configdir, filename)
        if os.path.isfile(full_filename):
            sys.stderr.write(
                "WARNING: Config already exists: {}\n".format(full_filename)
            )
            continue
        try:
            with open(full_filename, "w") as f:
                f.write(content)
            print("Config copied to {}".format(full_filename))
        except PermissionError as err:
            sys.stderr.write("{}\n".format(err))
            sys.exit(2)


def main() -> None:
    "main function"
    args = parseargs()

    if "show" in args.action:
        for filename, content in default_ini():
            print(f"-- {filename}\n{content}\n")

    if "user" in args.action:
        config_dir = caep.get_config_dir(CONFIG_ID, create=True)
        print("Config dir:", config_dir)
        save_config(config_dir)

    if "system" in args.action:
        save_config("/etc/")


if __name__ == "__main__":
    main()



import configparser
import os.path
from typing import List, Text

import addict

from act.scio.aliasregex import normalize
from act.scio.plugin import BasePlugin, Result
from act.scio.vocabulary import Vocabulary


def normalize_ta(name: Text, uppercase_abbr: List[Text]) -> Text:
    """Normalize TA names. Words are capitalized unless configured
    to be excempt"""

    res: Text = normalize(
        name,
        capitalize=True,
        uppercase_abbr=uppercase_abbr,
    )

    return res


def abbreviation_list(abbs: Text) -> List[Text]:
    """Split the abbrivation string on |, and return a list of
    abbreviations"""

    uppercase_abbr = [x.strip() for x in abbs.split("|")]
    uppercase_abbr = list(filter(bool, uppercase_abbr))

    return uppercase_abbr


class Plugin(BasePlugin):
    name = "threatactor"
    info = "Extracting references to known threat actors from a body of text"
    version = "0.2"
    dependencies: List[Text] = []

    async def analyze(self, nlpdata: addict.Dict) -> Result:

        ini = configparser.ConfigParser()
        ini.read([os.path.join(self.configdir, "threatactor_pattern.ini")])
        ini["threat_actor"]["alias"] = os.path.join(
            self.configdir, ini["threat_actor"]["alias"]
        )

        uppercase_abbr = abbreviation_list(
            ini["threat_actor"].get("uppercase_abbr", "")
        )

        vocab = Vocabulary(ini["threat_actor"])

        res = addict.Dict()

        res.ThreatActors = vocab.regex_search(
            nlpdata.content,
            normalize_result=(lambda x: normalize_ta(x, uppercase_abbr)),
            debug=self.debug,
        )

        return Result(name=self.name, version=self.version, result=res)



"""Helper function related to feed configuration"""

import argparse
import logging
import os
from enum import Enum
from typing import List, Optional, Text, Tuple

import caep

FeedType = Enum("FeedType", ["none", "partial", "full"])

XDG_CONFIG = os.path.expanduser(os.environ.get("XDG_CONFIG_HOME", "~/.config"))
XDG_CACHE = os.path.expanduser(os.environ.get("XDG_CACHE_HOME", "~/.cache"))


def get_args() -> argparse.Namespace:
    """initialize argument parser"""

    parser = argparse.ArgumentParser(description="pull feeds into html files")
    parser.add_argument(
        "-l", "--log", type=str, help="Which file to log to (default: stdout)"
    )
    parser.add_argument("--file-format", nargs="+", default="pdf doc xls csv xml")
    parser.add_argument(
        "--exclude-filenames", nargs="+", default="sitemap.xml robots.txt"
    )
    parser.add_argument(
        "--store-path",
        help=f"Location for stored files. Default {XDG_CACHE}/scio-feeds",
    )
    parser.add_argument("--proxy-string", help="Proxy to use for external queries")
    parser.add_argument("--ignore", type=str, help="file with ignore patterns")
    parser.add_argument(
        "--feeds",
        default=caep.get_config_dir("scio/etc/feeds.txt"),
        type=str,
        help=f"feed urls (one pr. line). Default: {XDG_CONFIG}/scio/etc/feeds.txt",
    )
    parser.add_argument(
        "--cache",
        default=caep.get_cache_dir("scio-feeds/cache.db"),
        help=f"sqlite db containing cached hashes. Default = {XDG_CACHE}/scio-feeds/cache.db",
    )
    parser.add_argument(
        "--tlp",
        help="Set TLP (RED, AMBER, GREEN, WHITE) on document upload. Default=WHITE",
        default="WHITE",
    )
    parser.add_argument(
        "--scio",
        help="Upload to scio engine API url. "
        + "Set to empty value to not upload files.",
        default="http://localhost:3000/submit",
    )
    parser.add_argument(
        "--stoplist",
        default=caep.get_config_dir("scio/etc/secstoplist.txt"),
        help="Provided own stoplist for text extraction",
    )
    parser.add_argument("--logfile")
    parser.add_argument("--loglevel", default="info")

    args: argparse.Namespace = caep.config.handle_args(
        parser, "scio/etc", "scio.ini", "feeds"
    )

    return args


def get_feed_url_from_feed_file_line(line: Text) -> Optional[Text]:
    """Extract the url portion from the feed file line"""

    if len(line) <= 2:
        logging.warning("Missconfigured feed file line: %s", line)
        return None

    feed_url = line[2:].strip()
    if not feed_url:
        logging.warning("Empty URL part of feed file line: %s", line)

    return feed_url


def get_feed_kind(line: Text) -> FeedType:
    """Extract the feed kind from a feed config line"""

    if len(line) < 2:
        return FeedType.none
    type_text = line[:2]
    if type_text == "p ":
        return FeedType.partial
    if type_text == "f ":
        return FeedType.full

    return FeedType.none


def parse_feed_file(filename: Text) -> Tuple[List[Text], List[Text]]:
    """Parse feed file, split feeds into partial and full feeds
    (lines starting with 'f ' and 'p ')"""

    full_feeds = []
    partial_feeds = []

    for linenum, feed_line in enumerate(open(os.path.expanduser(filename))):
        feed_type = get_feed_kind(feed_line)
        feed_url = get_feed_url_from_feed_file_line(feed_line)
        if not feed_url:
            continue

        if feed_type == FeedType.none:
            logging.error("Unable to parse line %s [%s]", linenum + 1, feed_line)
            continue

        if feed_type == FeedType.full:
            full_feeds.append(feed_url)
        if feed_type == FeedType.partial:
            partial_feeds.append(feed_url)

    return full_feeds, partial_feeds



#!/usr/bin/env python3
"""Copyright 2019 mnemonic AS <opensource@mnemonic.no>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

---
Program to check the feed output directory of feed_downloader.py
Any downloaded content not found in the cache is handled.
The .meta files associated with the .html files from the feed download is
kept to preserve information about source and original titles as well as
feed time and date stamps.
This files builds a document understood by SCIOs 'doc' work queue.
This utility is not meant to handle the files downloaded from links _in_ the
feed as these document does not share any of the meta-data associated with
the feed entries. These files need to be handled separately (using SCIOs own
submit utility.
"""

import argparse
import base64
import hashlib
import json
import logging
import os
import sqlite3
from typing import IO, Dict, List, Text, Tuple

import magic  # type: ignore
import requests

from act.scio.config import get_cache_dir

LOGGER = logging.getLogger("root")


class CandidateFile:
    """
    CandidateFile holds the metadata related to an .html file
    describing when the feed was published etc.
    """

    def __init__(self, filename: Text, my_metadata):

        LOGGER.debug("Creating CandidateFile %s", filename)

        self.filename = os.path.abspath(filename)
        self.metadata = my_metadata

        self.mime = magic.Magic(mime=True)
        self._sha256 = None

    def uploadable(self):
        """Check that the file content is part of a list of valid mime-types"""

        # Some file endings we need to upload no matter what (typicaly files
        # that is text type files.
        uploadable_file_endings = [".xml", ".csv", ".html", ".txt", ".json"]
        for file_ending in uploadable_file_endings:
            if file_ending in self.filename:
                return True

        # if not, we assume .docx and .pdf with friends actually has a mime
        # type starting in with application.
        return self.mime.from_file(self.filename).startswith("application")

    def sha256(self):
        """Compute the sha256 if it not allready computed, return the value"""

        if not self._sha256:  # compute sha256 only when needed and only once.
            LOGGER.debug("Computing SHA256 of %s", self.filename)
            with open(self.filename, "rb") as content_file:
                content = content_file.read()
                self._sha256 = hashlib.sha256(content).hexdigest()

        return self._sha256


class Cache:
    """Cache handles the caching database logic"""

    def __init__(self, filename="upload.sqlite"):
        """Initiate database, creating connection to file"""

        LOGGER.info("Connecting to %s", filename)

        self.conn = sqlite3.connect(filename)

        self.conn.execute(
            """
        CREATE TABLE IF NOT EXISTS upload (
            id integer PRIMARY KEY,
            filename text NOT NULL,
            sha256 text NOT NULL,
            description text)
        """
        )

    def uploaded(self, sha256):
        """Check if a particular digest is allready uploaded. Returns
        True/False"""

        sql = "SELECT * FROM upload WHERE sha256 = ?"

        cur = self.conn.execute(sql, (sha256,))

        if cur.fetchall():
            LOGGER.debug("Query for %s returns True", sha256)
            return True

        LOGGER.debug("Query for %s returns False", sha256)
        return False

    def insert(self, filename, sha256, description=""):
        """insert a new file in the metadata cache"""

        sql = "INSERT INTO upload(filename, sha256, description) VALUES(?,?,?)"
        LOGGER.debug(
            "Inserting %s, %s, %s into database", filename, sha256, description
        )
        self.conn.execute(sql, (filename, sha256, description))
        self.conn.commit()

    def info(self, sha256):
        """Get stored info about a digest. Returns a list of Dictionaries"""

        sql = (
            "SELECT filename, sha256, description FROM upload WHERE sha256 = ?"  # NOQA
        )

        cur = self.conn.execute(sql, (sha256,))

        results = cur.fetchall()
        LOGGER.debug("Found %d resultsults for %s", len(results), sha256)

        result_dictionaries = []

        for result in results:
            key_value_pairs = list(zip(["filename", "sha256", "description"], result))
            result_dictionaries.append(dict(key_value_pairs))

        return result_dictionaries


def init() -> argparse.Namespace:
    """initialize argument parser"""

    parser = argparse.ArgumentParser(description="Upload html to Scio")
    parser.add_argument(
        "-l", "--log", type=str, help="Which file to log to (default: stdout)"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="Log level INFO")
    parser.add_argument("--debug", action="store_true", help="Log level DEBUG")
    parser.add_argument(
        "--cache",
        type=str,
        help=(
            "Which database used for caching allready "
            + "uploaded files (default: ~/.cache/scio-feeds/upload.db)"
        ),
    )
    parser.add_argument(
        "--scio",
        type=str,
        default="http://localhost:3000/submit",
        help="URL to scio for submit (default=http://localhost:3000)",
    )
    parser.add_argument(
        "directories",
        metavar="DIR",
        type=str,
        nargs="*",
        help="Which directories to scan",
    )

    args = parser.parse_args()

    if not args.directories:
        args.directories = [
            os.path.join(get_cache_dir("scio-feeds", create=True), "download")
        ]

    if not args.cache:
        args.cache = os.path.join(get_cache_dir("scio-feeds", create=True), "upload.db")

    return args


def metadata(file_pairs: List[Tuple[Text, Text]]) -> List[Tuple[Text, Dict]]:
    """Takes a list of pairs (.html, .meta), opens the .meta file,
    parses the content and returns a list of pairs (.html, dict(meta))"""

    res = []

    for html, meta in file_pairs:
        if not os.path.isfile(html):
            LOGGER.warning("File not found %s (skipping)", html)
            continue
        if not os.path.isfile(meta):
            LOGGER.warning("File not found %s, skipping %s", meta, html)
            continue

        LOGGER.debug("Starting reading metadata : %s", meta)
        with open(meta, "r") as metadata_file:
            my_metadata = json.load(metadata_file)

        res.append((html, my_metadata))

    return res


def get_files(directories: List[Text]) -> List[CandidateFile]:
    """In a directory, get a listing of all .html files, pair
    them with the correct .meta file and build a list of
    CandidateFile object containing the path to the .html content
    file and the parsed meta data"""

    if not directories:
        LOGGER.warning("Directories are emtpy")
        return []

    for directory in directories:
        LOGGER.debug("Scanning %s", directory)
        files = os.listdir(directory)
        files = [os.path.join(directory, x) for x in files]

        res = [CandidateFile(cf, {}) for cf in files]

    return res


def read_as_base64(obj: IO) -> Text:
    """Create a base64 encoded string from a file like object"""

    encoded_bytes = base64.b64encode(obj.read())
    return encoded_bytes.decode("ascii")


def to_scio_submit_post_data(obj: IO, file_name: Text) -> Dict[Text, Text]:
    """Take a file like object, and return a dictionary on the correct form for
    submitting to the SCIO API"""

    return {"content": read_as_base64(obj), "filename": file_name}


def upload(args: argparse.Namespace) -> None:
    """entry point"""

    submit_cache = Cache(args.cache)

    candidates = get_files(args.directories)

    LOGGER.info("Found %d files", len(candidates))

    for candidate in candidates:

        partial_feed = candidate.metadata.get("partial_feed", False)
        if partial_feed:
            LOGGER.info("Partial feed: %s", candidate.filename)  # NOQA
            hexdigest = hashlib.sha256(
                candidate.metadata["link"].encode("utf-8")
            ).hexdigest()
            LOGGER.info("Partial feed: %s", hexdigest)  # NOQA
        else:
            hexdigest = candidate.sha256()

        if not submit_cache.uploaded(hexdigest):
            LOGGER.debug("submit %s", candidate.filename)
            submit_cache.insert(
                candidate.filename,
                hexdigest,
                candidate.metadata.get("creation-date", "NA"),
            )
            my_metadata = candidate.metadata
            my_metadata["filename"] = candidate.filename
            if candidate.uploadable():
                with open(candidate.filename, "rb") as file_h:
                    post_data = to_scio_submit_post_data(file_h, candidate.filename)
                    my_metadata.update(post_data)
                    session = requests.Session()
                    session.trust_env = False
                    session.post(args.scio, json=my_metadata)
            else:
                LOGGER.info(
                    "Not uploading %s (wrong mimetype)", candidate.filename
                )  # NOQA


def main() -> None:
    args = init()
    logformat = "%(asctime)-15s [%(filename)s:%(lineno)s - %(funcName)20s() ] %(message)s"  # NOQA

    logcfg = {
        "format": logformat,
        "level": logging.WARN,
    }

    if args.verbose:
        logcfg["level"] = logging.INFO

    if args.debug:
        logcfg["level"] = logging.DEBUG

    if args.log:
        logcfg["filename"] = args.log

    logging.basicConfig(**logcfg)

    try:
        upload(args)
    except IOError as err:
        LOGGER.error(err)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
This module contains function to convert an alias config file an/or an alias into
regular expressions for matching purposes"""

import re
import sys
from typing import Set, Text


def alias_set_from_config(config_file_name: Text) -> Set[Text]:
    """Read and parse a alias config file, add all aliases to a flat set"""

    alias_set = set()
    for line in open(config_file_name):
        pri, rest = re.split(r"(?<!\\):", line, maxsplit=1)
        aliases = re.split(r"(?<!\\),", rest)
        aliases.append(pri)
        aliases = [alias.strip() for alias in aliases if alias.strip() != ""]
        for alias in aliases:
            alias_set.add(alias)
    return alias_set


def regex_from_alias(alias: Text) -> Text:
    """convert a alias to a more general form using regex. All "breaks" in the
    alias (camlecase, numbers, underscores etc) is allowed to be "as is", a set
    of whitespace (one or more) and also -_/. to allow for different
    conventions in writing aliases"""

    def camel_case_break(alias: Text, i: int) -> bool:
        """Detect transistion from lower case to uppper case"""

        if i < 0:
            raise ValueError("Negative index not allowed")
        if i == 0:
            return False
        if not alias[i].isupper():
            return False
        if not alias[i - 1].islower():
            return False
        return True

    def alpha_to_digit_break(alias: Text, i: int) -> bool:
        """Detect transistion from letters to digist"""

        if i < 0:
            raise ValueError("Negative index not allowed")
        if i == 0:
            return False
        if not alias[i].isdigit():
            return False
        if not alias[i - 1].isalpha():
            return False
        return True

    tmp_re: Text = r"\b("  # start with word boundry
    for i, c in enumerate(alias):
        # transistions from lower to upper case and from letter to number may
        # also be written with a space.
        if camel_case_break(alias, i) or alpha_to_digit_break(alias, i):
            tmp_re += r"\s?[- _.]?"
        # Any space may or may not be there in text (som concat and use lower
        # to upper)
        if c.isspace():
            tmp_re += r"\s?[- _.]?"
        elif c.isdigit():
            tmp_re += r"\d"
        else:
            tmp_re += c.lower()

    tmp_re += r")\b"

    return tmp_re


def normalize(
    name: Text,
    space_before_numbers=True,
    space_before_capitalized=True,
    remove_non_alphanumeric=True,
    remove_multiple_whitespace=True,
    lower=True,
    upper=False,
    capitalize=False,
    uppercase_abbr=None,
) -> str:
    """

    Args:
    space_before_numbers (bool): insert space before numbers
    space_beofre_capitalized (bool): insert space before capitalized letters
    remove_non_alphanumeric (bool): remove charactes except alphanumer and space
    remove_multiple_whitespace (bool): replace multiple whitespace with a single space
    lower (bool): lowercase
    upper (bool): uppercase
    capitalize (bool): uppercase all first characters in words
    uppercase_abbr (list[str]): list of abbrevations to uppercase

    The transformation in ran in the same order as the arguments are specified,
    so you can use lower=True and capitalize=True to transform sOmeThing -> Something
    """

    if not uppercase_abbr:
        uppercase_abbr = []

    if space_before_numbers:
        # Replace "APT27" with "APT 27"
        name = re.sub(r"([A-Za-z])(\d)", r"\1 \2", name)

    if space_before_capitalized:
        # Replace "winntiGroup" with "winnti group"
        name = re.sub(r"([a-z])([A-Z])", r"\1 \2", name)

    if remove_non_alphanumeric:
        # Replace "APT-27" with "APT 27"
        name = re.sub(r"[^a-zA-Z0-9 ]+", " ", name, 0)

    if remove_multiple_whitespace:
        # Replace multiple whitespaces with a single whitespace
        name = re.sub(r"\s{2,}", " ", name)

    if lower:
        name = name.lower()

    if upper:
        name = name.upper()

    if capitalize:
        # https://stackoverflow.com/questions/6251463/regex-capitalize-first-letter-every-word-also-after-a-special-character-like-a
        name = re.sub(r"(((?<=\s)|^|-)[a-z])", lambda x: x.group().upper(), name)

    for abbr in uppercase_abbr:
        name = re.sub(abbr, abbr.upper(), name, 0, re.IGNORECASE)

    return name


def get_reg_ex_set(config_file_name: Text) -> Set[Text]:
    """Helper function to take a config file, parse it and create regular
    expressions from the aliases contained within. The aliases is returned in
    a set og regex strings"""

    regex_set = set()

    alias_set = alias_set_from_config(config_file_name)
    for alias in alias_set:
        if alias.isdigit():
            sys.stderr.write(
                f"WARNING: Unable to create regex from all digit alias {alias}\n"
            )
            continue
        regex_set.add(regex_from_alias(alias))

    return regex_set


if __name__ == "__main__":
    for regex in sorted(list(get_reg_ex_set("aliases.cfg"))):
        print(regex)



#!/usr/bin/env python3
"""Copyright 2019,2020 mnemonic AS <opensource@mnemonic.no>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

---
Program to check the feed output directory of feed_downloader.py
Any downloaded content not found in the cache is handled.
The .meta files associated with the .html files from the feed download is
kept to preserve information about source and original titles as well as
feed time and date stamps.
This files builds a document understood by SCIOs 'doc' work queue.
This utility is not meant to handle the files downloaded from links _in_ the
feed as these document does not share any of the meta-data associated with
the feed entries. These files need to be handled separately (using SCIOs own
submit utility.
"""

import logging
import sqlite3
from typing import Dict, List, Text


class Cache:
    """Cache handles the caching database logic"""

    def __init__(self, filename: Text = "upload.sqlite"):
        """Initiate database, creating connection to file"""

        logging.info("Connecting to %s", filename)
        self.conn = sqlite3.connect(filename)
        try:
            # create table if it does not exsit
            self.conn.execute(
                """CREATE TABLE upload (
                                              id integer PRIMARY KEY,
                                              filename text NOT NULL,
                                              sha256 text NOT NULL,
                                              description text);
                              """
            )
        except sqlite3.OperationalError:
            pass

    def contains(self, sha256: Text) -> bool:
        """Check if a particular digest is allready uploaded. Returns
        True/False"""

        sql = "SELECT * FROM upload WHERE sha256 = ?"

        cur = self.conn.execute(sql, (sha256,))

        if cur.fetchall():
            logging.debug("Query for %s returns True", sha256)
            return True

        logging.debug("Query for %s returns False", sha256)
        return False

    def insert(self, filename: Text, sha256: Text, description: Text = "") -> None:
        """insert a new file in the metadata cache"""

        sql = "INSERT INTO upload(filename, sha256, description) VALUES(?,?,?)"
        logging.debug(
            "Inserting %s, %s, %s into database", filename, sha256, description
        )
        self.conn.execute(sql, (filename, sha256, description))
        self.conn.commit()

    def info(self, sha256: Text) -> List[Dict]:
        """Get stored info about a digest. Returns a list of Dictionaries"""

        sql = (
            "SELECT filename, sha256, description FROM upload WHERE sha256 = ?"  # NOQA
        )

        cur = self.conn.execute(sql, (sha256,))

        results = cur.fetchall()
        logging.debug("Found %d resultsults for %s", len(results), sha256)

        result_dictionaries = []

        for result in results:
            key_value_pairs = list(zip(["filename", "sha256", "description"], result))
            result_dictionaries.append(dict(key_value_pairs))

        return result_dictionaries
